/*
	Advanced datatypes by Catid@pacbell.net
*/

#ifndef DATATYPES_H
#define DATATYPES_H


typedef unsigned char	BYTE;
typedef unsigned char	Uint8;
typedef unsigned short	Uint16;
typedef unsigned long	Uint32;
typedef short			Sint16;
typedef long			Sint32;
typedef unsigned int	uint;
typedef unsigned short	ushort;
typedef unsigned char	uchar;

union _regs
{
	struct
	{
		unsigned int eax, ebx, ecx, edx, esi, edi, ebp, esp;
	} d;
	struct
	{
		unsigned short ax, hax, bx, hbx, cx, hcx, dx, hdx, si, hsi, di, hdi, bp, hbp, sp, hsp;
	} x;
	struct
	{
		unsigned char al, ah, hax[2], bl, bh, hbx[2], cl, ch, hcx[2], dl, dh, hdx[2], rest[16];
	} h;
};


Uint32 getLong(BYTE * Message, const Uint32 Offset);		// Get a LONG from a BYTE * field

Uint16 getShort(BYTE * Message, const Uint32 Offset);		// Get a SHORT from a BYTE * field

Uint32 getLong(char * Message, const Uint32 Offset);		// Get a LONG from a char * field

Uint16 getShort(char * Message, const Uint32 Offset);		// Get a SHORT from a char * field

BYTE getByte(char * Message, const Uint32 Offset);			// Get a BYTE from a char * field


#define TEMPLATE template <class AnonymousStruct>


TEMPLATE class _listnode			// Linked list node
{
public:
	AnonymousStruct * item;									// This item
	_listnode		* next;									// Next item in the list
	_listnode		* last;									// Last item in the list

	_listnode(AnonymousStruct * nItem);						// Create item
	~_listnode();											// Destroy item
};


TEMPLATE class _linkedlist			// Linked list
{
public:
	_listnode <AnonymousStruct> * head;						// First item
	_listnode <AnonymousStruct> * tail;						// Last item

	void prepend(AnonymousStruct * item);					// Stick it in front
	void append(AnonymousStruct * item);					// Stick it in back
	void kill(_listnode <AnonymousStruct> * node,
			  bool nullify);								// Kill node
	void kill(AnonymousStruct * item,
			  bool nullify);								// Search and destroy
	void insertAfter(_listnode <AnonymousStruct> * node,
					 AnonymousStruct * item);				// Stick it after
	void insertBefore(_listnode <AnonymousStruct> * node,
					  AnonymousStruct * item);				// Stick it before
	void clear(bool nullify);								// Nuke the list

	Sint32 total;											// Total items

	_linkedlist();											// Initialize everything to zero
	~_linkedlist();											// Clear list without nullifying items
};


TEMPLATE class _jumptable			// Jump table
{
	void (*RPC [256])(AnonymousStruct *);					// "Remote Procedure Call" implementation

public:
	_jumptable();											// Initialize to zero

	void clear();											// Reset list
	void add(BYTE index,
			 void (*)(AnonymousStruct *));					// Create a handler
	void kill(BYTE index);									// Delete a handler
	bool call(BYTE index,
			  AnonymousStruct * params);					// Call a handler (false == unhandled)
};


TEMPLATE class _referencetable
{
	AnonymousStruct * LIST[256];

public:
	_referencetable();										// Clear list

	void clear();											// Zero list memory
	void add(BYTE index,
			 AnonymousStruct * data);						// Add an item
	void kill(BYTE index);									// Nuke an item
	AnonymousStruct * get(BYTE index);						// Get an item
};

#endif	//DATATYPES_H
